#!/bin/sh

# download and package firefox-esr for raspi
# 1st version

[ $(id -u) -ne 0 ] && exec sudo -A ${0} ${@}

export TEXTDOMAIN=install_firefox_gui.sh
export OUTPUT_CHARSET=UTF-8

CWD=`pwd`

# func strings
IMSG=$(gettext 'Please wait... checking latest versions')
ERR_TITLE=$(gettext 'Install Firefox-ESR - message')
EX=$(gettext ' already exists in /mnt/home. Load it or move it! Exiting.')
ERR_CON=$(gettext "There seems to be a problem connecting to the Raspbian servers.")
IS_RET=$(gettext 'is already retrieved')
ESFS=$(gettext 'There was a problem loading ')
OKSFS1=$(gettext ' is located in ')
OKSFS2=$(gettext '. You can choose to save this for later if you wish. It will be loaded when you close this window.')
SMSG=$(gettext ' is successfully installed! It is recommended that you run this program once a week to keep firefox updated.')
SMSG_NOINST=$(gettext ' is successfully downloaded and packaged however you did not install it. Save it or install with sfs_load. It is located in ')
ARMSG=$(gettext 'The ar utility is not installed. We will attempt to use dpkg-deb.')
DPMSG=$(gettext 'The dpkg-deb utility is not installed. Exiting')
WMSG=$(gettext 'Please wait while the package archive is unpacked...')
MSG1=$(gettext "You can download the latest Firefox ESR browser and create and install a squash filesystem (SFS) Puppy package.")
MSG2=$(gettext "Just hit the Firefox button and the download will start")
PETREM=$(gettext "Firefox is already installed as a PET package. Please uninstall with Puppy Package Manager and save session.")
UPTODATE=$(gettext "Firefox is already up to date")

. /etc/rc.d/PUPSTATE

# gui for messages
init_msg() {
	MSG="$1" 
	[ -z "$2" ] && GTKICON=no || GTKICON=$2
export ERR='<window title="'$ERR_TITLE'" icon-name="gtk-${GTKICON}">
	<vbox>
	'"`/usr/lib/gtkdialog/xml_info fixed /usr/share/pixmaps/dl_firefox.svg 60 "$MSG"`"' 
	  <frame>
	    <hbox homogeneous="true">
			<button ok></button>
		</hbox>
	  </frame>
	</vbox>
</window>'
. /usr/lib/gtkdialog/xml_info gtk
gtkdialog -p ERR -c 
[ "$2" = 'yes' ] || exit 1
}

# check if an sfs is loaded
is_sfs_loaded() {
	local B_SFS_LOADED
	[ -f /tmp/sfs_load/loop_device_list ] && B_SFS_LOADED=`grep "${1}" /tmp/sfs_load/loop_device_list`
	if [ -z "$B_SFS_LOADED" ]; then
		return 1 #not loaded
	fi
	return 0
}


# check if sfs is in /mnt/home
check_sfs_exists() {
	BRS_FILE="$1"
	is_sfs_loaded $BRS_FILE && return 0
	if [ -f "/initrd/${PUP_HOME}/${BRS_FILE}" ];then
		#kill -9 $apid
		init_msg "$BRS_FILE $EX"
	fi
}

# unload
unload_sfs() {
	sfs_load
}

# download, extract and build funtions
func_dld() {
	( 
	cd $TMP
	urxvt -bg  green -fg blue --geometry 80x20+1+1 -e wget ${REPO}/pool/main/f/firefox-esr/${1}
	)
}

func_extract() {
	(
	cd $TMP
	if [ -z "$ARNOT" ];then
		ar p "$1" data.tar.xz > data.tar.xz
		tar xJf data.tar.xz -C $TMPDIR
	else
		dpkg-deb -x "$1" $TMPDIR
	fi
	)
}

func_mk_sfs() {
	urxvt -bg  green -fg blue --geometry 80x20+1+1 -e mksquashfs $TMPDIR "$1"
}

load_sfs() {
	sfs_load +"${1}"
	return $?
}

# trap - clean up
_trap_exit() {
	cd $CWD
	rm -rf $TMP/data.tar.xz $TMP/*.deb $TMPDIR
	killall yaf-splash >/dev/null 2>&1
	echo killed
}
trap _trap_exit EXIT

# dependencies

if ! type ar >/dev/null 2>&1 ; then
	init_msg "$ARMSG" yes
	ARNOT=1
	if ! type dpkg-deb >/dev/null 2>&1 ; then
		init_msg "$DPMSG"
	fi
fi

# check arch
arch=`uname -m`

case $arch in
	arm*)DEBARCH=armhf
		CARCH=$DEBARCH;;
	*)exit;;
esac

# check pet installed
grep -q '^firefox' /var/packages/user-installed-packages ; retu=$?
grep -q '^firefox' /var/packages/woof-installed-packages ; retw=$?
[ $retu -eq 0 -o $retw -eq 0 ] && init_msg "$PETREM"

# find icons
SFSICON=/usr/share/pixmaps/puppy/package_sfs.svg
ICONTHEME=`grep -i 'icon-theme' ~/.gtkrc-2.0|sed -e 's/^gtk-icon-theme-name = //' -e 's/\"//g'`
[ -e "/usr/share/icons/$ICONTHEME/48/mimetypes/application-x-squashfs-image.svg" ] &&\
	SFSICON="/usr/share/icons/$ICONTHEME/48/mimetypes/application-x-squashfs-image.svg"
	
# tmp - working directory -if small RAM we do it on disk
TMP=${TMP:-/tmp}
read  a mem z <<< `busybox free|grep '^Mem'`
[ $mem -lt 1048576 ] && TMP=/initrd${PUP_HOME}
TMPDIR=${TMP}/firefox$$
mkdir -p $TMPDIR

# check if installed ####
# but wait..
gtkdialog-splash -close never -bg CornflowerBlue -text "$IMSG" &
export apid=$!

# mirror director - https://www.raspbian.org/RaspbianMirrors
REPO=http://mirrordirector.raspbian.org/raspbian
CVER=''
SVER=''
FF=$(wget -q -O - ${REPO}/pool/main/f/firefox-esr/ | 
	          sed '/href/!d; /\.deb/!d; /dbg/d; /dev/d; /l10n/d; /\.tar\./d; /dsc$/d; s/.*href="//; s/".*//; s/%2[bB]/+/;' | grep 'firefox-esr' | tail -1 | sed 's/\.mirrorlist$//')
grep -q "$FF" /var/packages/Packages-raspbian-buster-main || \
	FF=`grep '^firefox-esr' /var/packages/Packages-raspbian-buster-main | grep 'Internet' | cut -d '|' -f8`
[ -z "$FF" ] && init_msg "$ERR_CON"
SVER=`echo $FF | grep -oE '[0-9].*esr'`
SVER=${SVER/esr/}

if type firefox >/dev/null 2>&1 ; then
	read a b CVER <<< `firefox --version`
fi

kill -9 $apid

[ "$SVER" = "$CVER" ] && init_msg "$UPTODATE"
SS=true
BGUI=''
# check sfs exists in /mnt/home and not loaded
SFS=firefox-esr-${SVER}-armhf
# this stops the program on the first match
check_sfs_exists ${SFS}.sfs
#check if sfs is installed
if is_sfs_loaded ${SFS} ;then
	ICON=$SFSICON
	ACT=unload
	PROG=sfs_load
	SS=false
	# add to main gui
	TBASE1=$(gettext ', an older version is installed. Uninstall it with ')
	TBASE2=$(gettext ' - button on left - and run this program again later.')
	BGUI='<hbox space-expand="true" space-fill="true">
			<button space-expand="false" space-fill="true">
			  <input file>'$ICON'</input>
			  <width>32</width><height>32</height>
			  <action function="exit">'$ACT'</action>
			</button>
			<text use-markup="true" space-expand="true" space-fill="false" justify="2">
			  <label>"<i>Firefox-esr '${CVER}${TBASE1}${PROG}${TBASE2}'</i>"</label>
			</text>
		</hbox>'
fi

[ -n "$BGUI" ] && BGUI='<vbox space-expand="true" space-fill="false">'$BGUI'</vbox>'
####

FFBUT='<button image-position="top" theme-icon-size="32" tooltip-text="Firefox ESR version '$SVER'">
			<label>Firefox</label>
			<variable>btn1</variable>
			<input file>/usr/share/pixmaps/firefox.svg</input>
			<sensitive>'$SS'</sensitive>
		</button>'

# gui
export DLG='<window title="Install Firefox ESR" icon-name="internet">
	<vbox width-request="500">
	'"`/usr/lib/gtkdialog/xml_info fixed /usr/share/pixmaps/firefox.svg 60 "$MSG1"`"' 
	  <frame>
	  '$BGUI'
	  <hbox homogeneous="true">
	    <text><label>'"$MSG2"'</label></text>
	  </hbox>
	  <hbox homogeneous="true">  
	  '$FFBUT'
	  </hbox>
	  </frame>

	  <hbox>
	    <button cancel></button>
	  </hbox>
	</vbox>
</window>'

. /usr/lib/gtkdialog/xml_info gtk
case $1 in
 -d)echo "$DLG";;
 *)eval $(gtkdialog -p DLG);;
esac

echo $EXIT
case $EXIT in
	unload)unload_sfs
	exec $0;;
	Firefox)
		SFS=$SFS
		func_dld $FF;;
	*)exit;;
esac

BSFS=${SFS}.sfs
# errors more critical here on in
gtkdialog-splash -close never -bg CornflowerBlue -text "$WMSG" &
export wpid=$!
set -e
cd $TMP # here we extract and create sfs
func_extract ${FF}
(
	cd $TMPDIR
	sed -i -e "s/Icon=.*/Icon=firefox.svg/" \
			-e "s/Exec.*/Exec=firefox /" \
			-e 's/Categories=.*/Categories=WebBrowser;/' \
			usr/share/applications/*.desktop
			
	cat > usr/bin/firefox <<EOF
#!/bin/sh

LD_LIBRARY_PATH="/usr/lib/firefox-esr:\$LD_LIBRARY_PATH"

FIREFOX="\$(which firefox)"
[ -x "\$FIREFOX.real" ] && exec "\$FIREFOX.real" "\$@"

exec firefox-esr "\$@"
EOF
	chmod 755 usr/bin/firefox
)
kill -9 $wpid

func_mk_sfs "$BSFS"
cd -
# make sure sfs package exists and load it
if [ -e ${TMP}/${BSFS} ] ; then
	init_msg "${BSFS} $OKSFS1 ${TMP} $OKSFS2" yes
	load_sfs ${TMP}/${BSFS} || init_msg "$ESFS ${TMP}/${BSFS}"
fi
# announce success
	if [ -f /tmp/sfs_load/loop_device_list ] && grep -q "${BSFS}" /tmp/sfs_load/loop_device_list ; then
		init_msg "Firefox ESR $SMSG" yes
	else
		init_msg "Firefox ESR $SMSG_NOINST $TMP" yes
	fi
# END
